"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/** Build Group By String
 *
 * @param {Array} value
 * @returns {string}
 */
function buildGroupByString(value) {
    if (value.length === 0)
        return "";
    return "ORDER BY " + value.filter((v, i, self) => self.indexOf(v, i) === i).join(", ");
}
/** Build Order By String
 *
 * @param {Record} value
 * @returns {String}
 */
function buildOrderByString(value) {
    if (Object.getOwnPropertyNames(value).length === 0)
        return "";
    let string = "ORDER BY ";
    for (const name in value) {
        string += name + " " + value[name] + ", ";
    }
    return string.substring(0, string.length - 1);
}
/** Build Query Filter String
 *
 * @param {QueryConstraints} constraints
 * @returns {string}
 */
function buildFilterString(constraints = {}) {
    const { groupBy, orderBy, limit, offset } = constraints;
    let output = "";
    if (groupBy)
        output += buildGroupByString(groupBy) + " ";
    if (orderBy)
        output += buildOrderByString(orderBy) + " ";
    if (limit)
        output += `LIMIT ${limit} `;
    if (offset)
        output += `OFFSET ${offset}`;
    return output;
}
/** Database Query Builder
 *
 * Currntly only works with Cloudflare D1Database
 */
class QueryBuilder {
    constructor(db, object) {
        this._db = db;
        this._obj = object;
    }
    get db() {
        if (this._db === undefined)
            throw new Error("No database connection!");
        return this._db;
    }
    /** Insert Value into Database
     *
     * @param {Object} value
     */
    async insert(value) {
        const [string, values] = this._obj.buildInsertValues(value);
        const query = `INSERT INTO ${this._obj.name}${string}`;
        await this.db.prepare(query).bind(...values).run();
    }
    /** Update Value in Database
     *
     * @param {Object} value
     * @param {ObjectDefaults} constraints
     */
    async update(value, constraints) {
        const [updateString, updateValues] = this._obj.buildUpdateValues(value);
        const [constraintString, constraintValues] = this._obj.buildConstraints(constraints);
        const query = `UPDATE ${this._obj.name} ${updateString} ${constraintString}`;
        await this.db.prepare(query).bind(...updateValues.concat(constraintValues)).run();
    }
    /** Delete Value in Database
     *
     */
    async delete(constraints) {
        const [string, values] = this._obj.buildConstraints(constraints);
        const query = `DELETE FROM ${this._obj.name} ${string}`;
        await this.db.prepare(query).bind(...values).run();
    }
    /** Get First Value from Database
     *
     * @param {ObjectDefaults} constraints
     */
    async get(constraints) {
        const [string, values] = this._obj.buildConstraints(constraints);
        const query = `SELECT * FROM ${this._obj.name} ${string}`;
        const result = await this.db.prepare(query).bind(...values).first();
        if (result === null)
            return null;
        return this._obj.run(result);
    }
    /** Get ALl Values from Database
     *
     * @param {ObjectDefaults} constraints
     */
    async getAll(constraints, filter) {
        const [string, values] = this._obj.buildConstraints(constraints);
        const filterString = buildFilterString(filter);
        const query = `SELECT * FROM ${this._obj.name} ${string} ${filterString}`;
        const { results, error } = await this.db.prepare(query).bind(...values).all();
        if (error)
            throw error;
        return results.map((v) => this._obj.run(v));
    }
}
exports.default = QueryBuilder;
