/// <reference types="./global.d.ts" />
/** /Context
 *
 * @author Alex Malotky
 */
import View, { RenderUpdate } from "../View";
import OutgoingResponse, { NodeResponse } from "./OutgoingResponse";
import IncomingRequest, { NodeRequeset } from "./IncomingRequest";
import Authorization from "../Authorization";
import { BodyData } from "..//BodyParser";
import DataObject, { TypeOf, ObjectProperties } from "../Validation";
import QueryBuilder from "./QueryBuilder";
export type { NodeRequeset, NodeResponse };
/** Context
 *
 * Wrapper Around Request/Response
 */
export default class Context {
    private _request;
    private _response;
    private _url;
    private _env;
    private _view;
    private _auth;
    private _path;
    private _search;
    private _params;
    /** Constructor
     *
     * @param request
     * @param response
     */
    constructor(request: NodeRequeset | Request, response: NodeResponse | undefined, env: Env, view?: View, auth?: Authorization);
    /** Request Getter
     *
     */
    get request(): IncomingRequest;
    /** Response Getter
     *
     */
    get response(): OutgoingResponse;
    /** Environment Getter
     *
     */
    get env(): Env;
    /** Url Getter
     *
     */
    get url(): URL;
    /** Method Getter
     *
     */
    get method(): string;
    /** Params Getter
     *
     */
    get params(): Map<string, string>;
    /** Search Getter
     *
     */
    get search(): Map<string, string>;
    /** Path Getter
     *
     */
    get path(): string;
    /** Path Setter
     *
     */
    set path(value: string);
    /** Set Status Code
     *
     * @param {number} value
     * @returns {this}
     */
    status(value: number): this;
    /** Write Chunk to Response
     *
     * @param {any} chunk
     * @returns {this}
     */
    write(chunk: any): this;
    /** End Response
     *
     */
    end(): void;
    /** Set Json Content
     *
     * @param {Object} object
     * @returns {this}
     */
    json(object: Object): this;
    /** Set Text Content
     *
     * @param {string} value
     * @returns {this}
     */
    text(value: string): this;
    /** Set HTML Content
     *
     * @param {string} value
     * @returns {this}
     */
    html(value: string): this;
    /** Expects Render
     *
     * Checks headers to test if request is expecting render object.
     *
     * @returns {boolean}
     */
    expectsRender(): boolean;
    /** Reunder Update Content
     *
     * @param {ContentUpdate} value
     * @returns {this}
     */
    render(value: RenderUpdate): this;
    /** Form Data Getter
     *
     */
    formData<P extends ObjectProperties>(expected?: DataObject<P>): Promise<TypeOf<DataObject<P>>>;
    formData(): Promise<BodyData>;
    /** Flush & Get Response
     *
     */
    flush(): Promise<Response | undefined>;
    /** Get Authorization
     *
     * @returns {Promise<User|null>}
     */
    getAuth(): Promise<User | null>;
    /** Set Authorization
     *
     * @returns {Promise<void>}
     */
    setAuth(user: User | null): Promise<void>;
    /** Redirect
     *
     * @param url
     * @param status
     */
    redirect(url: string | URL, status?: number): void;
    /** Database Query Builder
     *
     * @returns {QueryBuilder}
     */
    query<P extends ObjectProperties>(object: DataObject<P>): QueryBuilder<P>;
}
