"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_to_regexp_1 = require("path-to-regexp");
const Util_1 = require("../Util");
/** Single Routing Layer
 *
 */
class Layer {
    constructor() {
        this._path = "/";
        this._opts = {};
        this._prefix = "";
        switch (arguments.length) {
            case 0:
                throw new Error("No arguments passed to layer!");
            case 1:
                if (typeof arguments[0] !== "function")
                    throw new TypeError("Middleware must be a function!");
                this._handler = arguments[0];
                break;
            case 2:
                if (typeof arguments[0] !== "string")
                    throw new TypeError("Path must be a string!");
                if (typeof arguments[1] !== "function")
                    throw new TypeError("Middleware must be a function!");
                this._handler = arguments[1];
                this._path = arguments[0];
                break;
            default:
                if (typeof arguments[0] !== "string")
                    throw new TypeError("Path must be a string!");
                if (typeof arguments[1] !== "object")
                    throw new TypeError("Options must be an object!");
                if (typeof arguments[2] !== "function")
                    throw new TypeError("Middleware must be a function!");
                this._handler = arguments[2];
                this._path = arguments[0];
                this._opts = arguments[1];
                break;
        }
        if (this._path === "*") {
            this._shortcut = true;
            this._path += "path";
        }
        else if (this._path === "/") {
            this._shortcut = this._opts.end === false;
        }
        else {
            this._shortcut = false;
        }
        const { regexp, keys } = (0, path_to_regexp_1.pathToRegexp)(this._path, this._opts);
        this._regex = regexp;
        this._keys = keys;
    }
    /** Handle Routing
     *
     * @param {Context} context
     */
    async handle(context) {
        try {
            if (this.match(context)) {
                await this._handler(context);
            }
        }
        catch (e) {
            throw e;
        }
    }
    /** Match Routing
     *
     * @param {Context} context
     * @returns {boolean}
     */
    match(context) {
        if (this._shortcut) {
            return true;
        }
        const match = context.url.pathname.match(this._regex);
        if (match == null)
            return false;
        for (let index = 1; index < match.length; index++) {
            context.params.set(this._keys[index - 1].name, decodeURIComponent(match[index]));
        }
        context.path = context.url.pathname.replace(match[0], "");
        return true;
    }
    /** Path Getter
     *
     */
    get path() {
        return this._path;
    }
    /** Path Setter
     *
     */
    set path(value) {
        this._path = value;
        this._shortcut = false;
        value = (0, Util_1.joinPath)(this._prefix, this._path);
        const { regexp, keys } = (0, path_to_regexp_1.pathToRegexp)(value, this._opts);
        this._keys = keys;
        this._regex = regexp;
    }
    /** Get Total Path
     *
     * @returns {string}
     */
    totalPath() {
        return (0, Util_1.joinPath)(this._prefix, this._path);
    }
    /** Set Prefix
     *
     * @param {string} value
     */
    prefix(value) {
        this._prefix = value;
        value = (0, Util_1.joinPath)(value, this._path);
        const { regexp, keys } = (0, path_to_regexp_1.pathToRegexp)(value, this._opts);
        this._keys = keys;
        this._regex = regexp;
    }
    /** Update Options
     *
     * @param {PathToRegexpOptions} value
     */
    update(value) {
        this._opts = value;
        const { regexp, keys } = (0, path_to_regexp_1.pathToRegexp)(this._path, value);
        this._keys = keys;
        this._regex = regexp;
    }
}
exports.default = Layer;
