"use strict";
/** /View/RenderEnvironment/Util
 *
 * Front End Utility
 *
 * @author Alex Malotky
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getValueFrom = exports.hashObject = exports.getRouteInfo = exports.findOrCreateElement = void 0;
/** Find Or Create Element
 *
 * @param {string} name
 * @param {Array<string>} parents
 * @returns {HTMLElement}
 */
function findOrCreateElement(name, ...parents) {
    if (typeof name === "undefined") {
        return document.body;
    }
    else if (typeof name !== "string") {
        throw new TypeError(`Unknown type '${typeof name}' for query string`);
    }
    let node = document.querySelector(name);
    if (node)
        return node;
    node = findOrCreateElement(...parents);
    let attributes;
    let index = name.indexOf("[");
    if (index > 0) {
        attributes = name.replace(/.*?\[(.*?)='(.*?)'\]?/gm, "$1=$2 ").split(/\s+/gm) || [];
        name = name.substring(0, index);
    }
    let id;
    let className;
    index = name.indexOf("#");
    if (index >= 0) {
        id = name.substring(index + 1);
        name = name.substring(0, index);
    }
    index = name.indexOf(".");
    if (index >= 0) {
        className = name.substring(index + 1);
        name = name.substring(0, index);
    }
    if (name.trim().length === 0) {
        name = "div";
    }
    let newNode = document.createElement(name);
    if (id)
        newNode.id = id;
    if (className)
        newNode.className = className;
    if (attributes) {
        for (let string of attributes) {
            string = string.trim();
            if (string.length > 3) {
                const buffer = string.split("=");
                newNode.setAttribute(buffer[0], buffer[1]);
            }
        }
    }
    node.appendChild(newNode);
    return newNode;
}
exports.findOrCreateElement = findOrCreateElement;
/** Get Routing Info
 *
 * Returns both the pathname and anchor id from a Hypertext Reference string.
 *
 * @param {string} href
 * @returns {{path:string, anchor:string}}
 */
function getRouteInfo(href) {
    if (href instanceof URL) {
        return {
            path: href.pathname,
            anchor: href.hash.substring(1)
        };
    }
    let regex = "(https?://" + location.hostname;
    const port = location.port;
    if (port !== "80" && port !== "443")
        regex += ":" + port;
    href = href.replace(new RegExp(regex + ")"), "");
    const index = href.indexOf("#");
    if (index < 0) {
        return {
            path: href,
            anchor: ""
        };
    }
    else if (index === 0) {
        return {
            path: "",
            anchor: href
        };
    }
    return {
        path: href.substring(0, index),
        anchor: href.substring(index + 1)
    };
}
exports.getRouteInfo = getRouteInfo;
/** Hash Object
 *
 */
function hashObject(value) {
    let string;
    switch (typeof value) {
        case "number":
            return value;
        case "undefined":
            return 0;
        case "boolean":
            return Number(value);
        case "object":
            if (value === null)
                return 0;
            string = JSON.stringify(value);
            break;
        case "string":
            string = value;
            break;
        case "function":
            string = value.name;
            break;
        case "bigint":
        case "symbol":
            string = value.toString();
    }
    if (string.length === 0)
        return 0;
    let hash = 0;
    for (let i = 0; i < string.length; i++) {
        hash = ((hash << 5) - hash) + string.charCodeAt(i);
        hash |= 0;
    }
    return hash;
}
exports.hashObject = hashObject;
/** Get Value From Element
 *
 * @param {HTMLElement} element
 * @param {string} name
 * @returns {string}
 */
function getValueFrom(element, name) {
    const target = element.querySelector(`[name='${name}']`);
    if (target && target.value) {
        return target.value;
    }
    return "";
}
exports.getValueFrom = getValueFrom;
