"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Util_1 = require("./Util");
const Head_1 = __importDefault(require("./Head"));
const Util_2 = require("../../Util");
/** Front End Render Environment
 *
 */
class RenderEnvironment {
    /** Render Environment Constructor
     *
     */
    constructor() {
        this._routing = false;
        this._head = new Head_1.default();
        this._headHash = 0;
        this._bodyHash = 0;
        this._events = [];
    }
    /** Main Route Handler
     *
     * Returns value if there is an anchor to scroll too.
     *
     * @param {FormData} body
     * @returns {Promise<string|undefined>}
     */
    async handler(url = window.location.href, body) {
        this._routing = true;
        const { anchor, path } = (0, Util_1.getRouteInfo)(url);
        try {
            const data = await RenderEnvironment.fetch(path, { body });
            if (data.redirect) {
                return this.handler(data.redirect);
            }
            await this.update(data);
            window.history.pushState({}, "", url);
        }
        catch (e) {
            RenderEnvironment.error(e);
        }
        this._routing = false;
        if (this._delay) {
            const { url, body } = this._delay;
            this._delay = undefined;
            return this.handler(url, body);
        }
        return anchor;
    }
    /** Route to locale page.
     *
     * @param {string|URL} url
     * @param {FormData} body
     */
    route(url, body) {
        if (!this._routing) {
            this.clear();
            this.handler(url, body).then(anchor => {
                if (anchor)
                    this.scroll(anchor);
            });
        }
        else {
            this._delay = { url, body };
        }
    }
    /** Scroll to Element
     *
     * @param {string} id
     */
    scroll(id) {
        if (id) {
            const element = document.getElementById(id);
            if (element) {
                element.scrollIntoView();
            }
        }
    }
    /** Open External Link
     *
     * @param {string|URL} href
     */
    link(href) {
        window.open(href, '_blank', 'noopener,noreferrer');
    }
    /** Run Script
     *
     * @param script
     */
    run(script) {
        new Function("env", script)(this);
    }
    /** Add Event
     *
     * @param update
     */
    event(type, listener) {
        if (typeof type !== "string")
            throw new TypeError("Unknown event!");
        if (typeof listener !== "function")
            throw new TypeError("Unknown listener!");
        this._events.push({ type, listener });
        window.addEventListener(type, listener);
    }
    /** Update Environment
     *
     * @param {RenderUpdate} update
     */
    async update(update) {
        if (update.head) {
            await this.updateHead(update.head);
        }
        if (update.body) {
            this.updateBody(update.body);
        }
    }
    ////////////// Private Update Methods //////////////
    /** Update Head
     *
     * @param {HeadUpdate} update
     */
    async updateHead(update) {
        const hash = (0, Util_1.hashObject)(update);
        if (hash === this._headHash) {
            return console.info("Head didn't change!");
        }
        await this._head.update(update);
        this._headHash = hash;
    }
    /** Update Body
     *
     */
    updateBody(update) {
        const hash = (0, Util_1.hashObject)(update);
        const scripts = [];
        if (hash === this._bodyHash) {
            return console.info("Body didn't change!");
        }
        for (const id in update) {
            const element = document.getElementById(id);
            if (element) {
                RenderEnvironment.render(element, update[id]);
                const match = update[id].match(/<script.*?>.*?<\/script.*?>/gi);
                if (match) {
                    scripts.push(...match);
                }
            }
            else {
                console.warn(`Unable to find element with id '${id}'!`);
            }
        }
        this.scripts(scripts);
        this._bodyHash = hash;
    }
    /** Run Scripts
     *
     * @param {RegExpMatchArray} update
     */
    scripts(update) {
        for (let script of update) {
            this.run(script.replace(/^<script.*?>(.*?)<\/script.*?>$/gi, "$1"));
        }
    }
    /** Clear Events
     *
     */
    clear() {
        while (this._events.length > 0) {
            const { type, listener } = this._events.pop();
            window.removeEventListener(type, listener);
        }
    }
    ////////////// Static FunctionS //////////////
    /** Assign Render Content to Target
     *
     * @param {HTMLElement} target
     * @param {RenderContent} content
     */
    static render(target, content) {
        if (target.value) {
            target.value = content;
        }
        else {
            target.innerHTML = content;
        }
    }
    /** Fetch Wrapper
     *
     * @param {stirng|URL} url
     * @param {FetchOptions} opts
     * @returns {Promise<RenderUpdate>}
     */
    static async fetch(url, opts) {
        if (opts.headers === undefined)
            opts.headers = {};
        opts.headers[Util_2.HEADER_KEY] = Util_2.HEADER_VALUE;
        try {
            const response = await fetch(url, opts);
            if (response.headers.get(Util_2.HEADER_KEY) !== Util_2.HEADER_VALUE) {
                throw new Error("Did not recieve an update response!");
            }
            else if (response.headers.get("Content-Type") !== "application/json") {
                throw new Error("Did not recieve JSON response!");
            }
            const data = await response.json();
            if (data.redirect === undefined && data.head === undefined && data.body === undefined && data.update === undefined) {
                throw new Error("Recieved either an empty or invalid response!");
            }
            return data;
        }
        catch (e) {
            throw e;
        }
    }
    /** Display Error
     *
     * @param {any} value
     */
    static error(value) {
        const dialog = document.createElement("dialog");
        dialog.style.position = "absolute";
        dialog.style.width = "100%";
        dialog.style.height = "100%";
        dialog.style.top = "0";
        dialog.style.left = "0";
        dialog.style.display = "flex";
        dialog.style.alignItems = "center";
        dialog.style.justifyContent = "center";
        dialog.style.background = "rgba(0,0,0,0.5)";
        dialog.addEventListener("click", () => {
            document.body.removeChild(dialog);
        });
        const modal = document.createElement("div");
        modal.style.backgroundColor = "white";
        modal.style.padding = "5px";
        modal.addEventListener("click", (event) => {
            event.stopPropagation();
        });
        const header = document.createElement("h1");
        header.style.color = "red";
        header.style.textAlign = "center";
        header.textContent = "Error";
        modal.appendChild(header);
        const message = document.createElement("p");
        message.style.color = "red";
        message.style.textAlign = "center";
        message.textContent = String(value);
        modal.appendChild(message);
        const close = document.createElement("button");
        close.style.display = "block";
        close.style.margin = "0 auto";
        close.textContent = "Ok";
        modal.appendChild(close);
        close.addEventListener("click", () => {
            document.body.removeChild(dialog);
        });
        dialog.appendChild(modal);
        dialog.open = true;
        document.body.appendChild(dialog);
        console.error(value);
    }
}
exports.default = RenderEnvironment;
